from pyfiglet import figlet_format
from typing import List
from colorama import Fore, Style
import sys
import logging
from sylo.models import Durations, Config


logger = logging.getLogger(__name__)


config = Config()


class Display:
    def __init__(self, theme_color: str = config.theme_color):
        self.theme = self.color_map(theme_color)

    def col(self, string: str, color: str = None):
        if color:
            return f"{self.color_map(color)}{string}{Style.RESET_ALL}"
        else:
            return f"{self.theme}{string}{Style.RESET_ALL}"

    def print_timer(self, mode: str, timer_val: int, tasks: List = None):
        if mode == "work":
            self.print_header_small()
            self.print_tasks(tasks)
            print(
                f"{self.col('Ctrl-C to stop timer early.')}"
                "\n"
                f"\n{self.col('WORK', 'red')} "
                f"for {self.col(str(timer_val), 'yellow')} minutes"
            )
        else:
            print(
                f"Ctrl-C to stop timer early."
                f"\n{self.col('REST', 'green')} "
                f"for {self.col(str(timer_val), 'yellow')} minutes"
            )

    def print_summary_and_quit(self):
        sys.stdout.write("\033[K")
        print(
            f"{self.col('Press')} "
            f"{self.col('ENTER', 'yellow')}"
            f"{self.col('to stop timer.')}",
        )

    def print_insights(self):
        print(
            f"\n{self.col('Press')} "
            f"{self.col('ENTER', 'yellow')} "
            f"{self.col('to return to main menu.')}"
        )

    def print_bar_header(self):
        print(f"{self.col('Weekly Minutes', 'yellow')}")

    def print_heat_header(self):
        print(f"{self.col('Work Heatmap', 'yellow')}")

    def cursor(self):
        return self.col(">> ", "yellow")

    def print_options(self):
        print(
            "Additional commands;"
            f"\n{self.col('S', 'yellow')}{self.col('       --    Swap upcoming timer')}"
            f"\n{self.col('T', 'yellow')}{self.col('       --    Toggle display of tasks for today')}"
            f"\n{self.col('Y', 'yellow')}{self.col('       --    Toggle display of tasks for yesterday')}"
            f"\n{self.col('N', 'yellow')}{self.col('       --    Add new tasks for today')}"
            f"\n{self.col('I', 'yellow')}{self.col('       --    Show insights')}"
            f"\n{self.col('Q', 'yellow')}{self.col('       --    Quit')}"
        )

    def print_splash_prompt(self):
        print(
            f"{self.col('Press')} {self.col('ENTER', 'yellow')} {self.col('to start the next timer.')}"
        )

    def print_offer_options(self):
        print(f"{self.col('Press')} {self.col('H', 'yellow')} {self.col('for Help.')}")

    def print_add_task_or_quit(self):
        print(
            f"{self.col('ENTER', 'yellow')} "
            f"{self.col('to add a new task or')} "
            f"{self.col('Q', 'yellow')} "
            f"{self.col('to go back')}"
        )

    def print_new_task(self):
        print(f"{self.col('Input a new task')}")

    def print_another_task(self):
        print(f"{self.col('Input another? Y/N')}")

    def print_splash(self, durations: Durations, mode: str):
        print(
            f"Total {self.col('work', 'red')} today:           "
            f"{self.col(str(int(durations.total_work_mins)), 'yellow')} minutes"
            f"\nTotal {self.col('rest', 'green')} today:           "
            f"{self.col(str(int(durations.total_rest_mins)), 'yellow')} minutes"
            f"\nUpcoming timer:     {self.col(mode.upper(), self._set_mode_color(mode, durations))}"
            f"\n"
        )

    def print_splash_variants(
        self,
        durations: Durations,
        mode: str,
        is_options: bool = None,
        is_tasks: bool = None,
        tasks: List = None,
    ):
        logger.info(f"Splash. is_options: {is_options} is_tasks: {is_tasks}")
        self.print_header_small()
        self.print_splash(durations, mode)
        if is_options is True and is_tasks is False:
            self.print_options()
            self.print_splash_prompt()
        elif is_options is False and is_tasks is True:
            self.print_tasks(tasks)
            self.print_splash_prompt()
            self.print_offer_options()
        elif is_options is True and is_tasks is True:
            self.print_tasks(tasks)
            self.print_options()
            self.print_splash_prompt()
        else:
            self.print_splash_prompt()
            self.print_offer_options()

    def print_tasks(self, tasks: List = None):
        if tasks:
            print(f'{self.col(f"Tasks", "yellow")}')
            print("\n".join("".join(map(str, f"{t[0]} -- {t[1]}")) for t in tasks))
            print("\n")

    def print_header_small(self):
        print(self.col(self.ascii_header(string="SYLO", font="alligator", width=60)))

    @staticmethod
    def color_map(color: str):
        colors = {
            "red": Fore.RED,
            "lred": Fore.LIGHTRED_EX,
            "blue": Fore.BLUE,
            "lblue": Fore.LIGHTBLUE_EX,
            "green": Fore.GREEN,
            "lgreen": Fore.LIGHTGREEN_EX,
            "yellow": Fore.YELLOW,
            "magenta": Fore.MAGENTA,
            "white": Fore.WHITE,
            "black": Fore.BLACK,
        }
        return colors[color]

    @staticmethod
    def ascii_header(string: str, font: str, width: int):
        return figlet_format(string, font=font, width=width)

    @staticmethod
    def _set_mode_color(mode: str, durations: Durations):
        if mode == "rest":
            return durations.rest.bar_color
        else:
            return durations.work.bar_color



#!/usr/bin/env python
import sys
import logging
import time
import simpleaudio
from beepy import beep
from tqdm import tqdm
from colorama import Fore
from typing import List
import csv
from sylo.models import Durations, DataPersistence, get_tasks, ConfigFile
from sylo.definitions import (
    TASK_FILE_LOCATION,
    TODAY_STR,
    YESTERDAY_STR,
    WELCOME_CHOICES,
    COUNTDOWN_INCREMENTS,
    DATA_FILE_LOCATION,
    METRICS_BAR,
    METRICS_HEAT,
    DATA_FILE_CREATE,
    TASK_FILE_CREATE,
    HOME_DIR_CREATE,
    ONE_WEEK_AGO,
    ONE_YEAR_AGO,
)
from sylo.utils import (
    clear_screen,
    mins_to_secs,
    flip_mode,
    get_input,
    run_subprocess,
    check_for_file,
)
from sylo.display import Display


logger = logging.getLogger(__name__)
display = Display()
is_speed_mode = False
timer_mode = "work"
data_file = DataPersistence()
cursor = display.cursor()
initial_boot_flag = False


class Timer:
    def __init__(self, mode: str, durations: Durations, increments: int):
        self.durations = durations
        self.mode: str = mode
        self.increments = increments
        self.mins_passed = 0
        self.tasks = get_tasks(TODAY_STR)

        if self.mode == "work":
            self.mins = self.durations.work.mins
            self.secs = self.durations.work.secs
            self.bar_color = self.durations.work.bar_color
        else:
            self.mins = self.durations.rest.mins
            self.secs = self.durations.rest.secs
            self.bar_color = self.durations.rest.bar_color

    def _start_timer(self):
        timer_iterator = tqdm(
            total=self.secs,
            position=0,
            leave=False,
            desc=display.print_timer(self.mode, self.mins, self.tasks),
            unit=" seconds",
            bar_format="{l_bar}%s{bar}%s{r_bar}"
            % (display.color_map(self.bar_color), Fore.RESET),
            smoothing=True,
            ncols=70,
        )
        try:
            while self.secs > 0:
                time.sleep(1)
                self.secs -= self.increments
                timer_iterator.update(self.increments)
                if self.secs % 60 == 0:
                    self.mins_passed += 1
                    logger.debug(f"{self.mode} Timer loop: secs remaining: {self.secs}")

        except KeyboardInterrupt:
            print("\nQuitting early..")
            logger.info(f"Ctrl-C quit with {self.secs} remaining")
            timer_iterator.close()
            return False, self.mins_passed
            pass
        logger.info(f"{self.mode} timer loop finished")
        timer_iterator.close()
        return True, self.mins_passed

    def start_countdown(self):
        logger.info(f"{self.mode} timer started")
        complete, mins_passed = self._start_timer()
        return complete, mins_passed


class Sound:
    def __init__(self, audio_path: str = "dummy"):
        self.sound_file = audio_path
        self.beep_needed = False
        self.initialised_file = None

    def initialise(self):
        logger.info(f"Initialising audio: {self.sound_file}")
        try:
            self.initialised_file = simpleaudio.WaveObject.from_wave_file(
                self.sound_file
            )
            logger.info(f"Custom audio successfully initialised: {self.sound_file}")
        except FileNotFoundError:
            logger.info(f"Custom audio not found: {self.sound_file}")
            self.beep_needed = True

    def play_sound(self):
        if self.beep_needed is False:
            play_obj = self.initialised_file.play()
            logger.info("Played audio file")
            play_obj.wait_done()
            logger.info("Waiting audio file")

        else:
            logger.info("Playing beep")
            beep("ready")


class Insights:
    def __init__(self):
        self.data_file = DATA_FILE_LOCATION
        self.heatmap_start_date = ONE_YEAR_AGO
        self.bar_start_date = ONE_WEEK_AGO

    def print_bar(self):
        global initial_boot_flag
        display.print_header_small()
        display.print_bar_header()
        if initial_boot_flag is False:
            logger.info(f"Running ${self._bar_chart()}")
            run_subprocess(self._bar_chart())
        else:
            print("No data available, complete some segments and check later.")

    def print_heat(self):
        logger.info(f"Running ${self._heat_map()}")
        display.print_heat_header()
        run_subprocess(self._heat_map())

    def _bar_chart(self):
        return METRICS_BAR % (self.data_file, self.bar_start_date)

    def _heat_map(self):
        return METRICS_HEAT % (self.heatmap_start_date, self.data_file)


class Task:
    def __init__(self):
        self.description = None
        self.date = TODAY_STR
        self.complete = False

    def write_task_to_file(self):
        fields = [self.date, self.description, self.complete]
        with open(TASK_FILE_LOCATION, "a") as f:
            writer = csv.writer(f)
            writer.writerow(fields)


class Sylo:
    def __init__(
        self,
        durations: Durations,
        sound_obj: any,
        mode: str,
        show_options: bool = False,
        show_tasks: bool = False,
        tasks: List = None,
        tasks_date: str = None,
        speed_mode: bool = is_speed_mode,
    ):
        self.durations = durations
        self.sound_obj = sound_obj
        self.mode = mode
        self.show_options = show_options
        self.show_tasks = show_tasks
        self.tasks = tasks
        self.tasks_date = tasks_date
        self.speed_mode = speed_mode
        self.increments = 1

    def on_boot(self):
        clear_screen()
        if self.speed_mode is True:
            self.increments = 30
        else:
            self.increments = COUNTDOWN_INCREMENTS
        logger.debug(f"Running in {self.increments} second increments")

    def splash(self, timer: Timer):
        display.print_splash_variants(
            timer.durations,
            mode=self.mode,
            is_options=self.show_options,
            is_tasks=self.show_tasks,
            tasks=self.tasks,
        )

    def task_menu(self, date: str = None):
        self.show_tasks ^= True
        self.tasks = get_tasks(date)
        self.input_loop()

    def new_task_menu(self, date: str):
        clear_screen()
        task = Task()
        self.tasks = get_tasks(date)
        display.print_header_small()
        display.print_tasks(self.tasks)
        display.print_add_task_or_quit()
        usr_input = input(cursor)
        if usr_input.lower() == "q":
            self.input_loop()
        if usr_input == "":
            display.print_new_task()
            task.description = input('Description >> ')
            task.write_task_to_file()
            display.print_another_task()
            another_yn = input(cursor)
            if another_yn.lower() == "y":
                self.new_task_menu(date)
            else:
                self.input_loop()

    def switch_modes(self):
        logger.info("Switch requested")
        self.mode = flip_mode(self.mode)
        self.input_loop()

    def show_help_menu(self):
        logger.info("Help requested")
        self.show_options ^= True
        self.input_loop()

    def show_insights_menu(self):
        clear_screen()
        metrics = Insights()
        metrics.print_bar()
        metrics.print_heat()
        display.print_insights()
        input(cursor)
        self.input_loop()

    def timer_loop(self, timer: Timer):
        clear_screen()
        global initial_boot_flag
        logger.info("User pressed ENTER")
        if self.mode == "rest":
            complete, secs_passed = timer.start_countdown()
            if complete is True:
                self.sound_obj.play_sound()
                self.durations.total_rest_mins += secs_passed
            else:
                self.durations.total_rest_mins += secs_passed
            self.mode = flip_mode(self.mode)
            data_file.data_refresh(self.durations)
            clear_screen()
        else:
            complete, secs_passed = timer.start_countdown()
            if complete is True:
                self.sound_obj.play_sound()
                self.durations.total_work_mins += self.durations.work.mins
            else:
                self.durations.total_work_mins += secs_passed
            self.mode = flip_mode(self.mode)
            data_file.data_refresh(self.durations)
            clear_screen()
        logger.info("Durations model set")
        logger.info(f"Work time: {self.durations.total_work_mins}")
        logger.info(f"Work time: {self.durations.total_rest_mins}")
        initial_boot_flag = False
        self.input_loop()

    @staticmethod
    def quit_app():
        logger.info("Quit requested")
        clear_screen()
        sys.exit()

    def input_loop(self):
        self.on_boot()
        timer = Timer(self.mode, self.durations, self.increments)
        self.splash(timer)
        response = get_input(cursor)
        logger.info(f"User response: {response}")
        while response.lower() not in WELCOME_CHOICES:
            self.show_options = True
            logger.info(f"Rejected user response of {response}")
            self.input_loop()
        while response.lower() == "q":
            self.quit_app()
        while response.lower() == "i":
            self.show_insights_menu()
        while response.lower() == "s":
            self.switch_modes()
        while response.lower() == "h":
            self.show_help_menu()
        while response.lower() == "t":
            self.task_menu(TODAY_STR)
        while response.lower() == "y":
            self.task_menu(YESTERDAY_STR)
        while response.lower() == "a":
            self.task_menu()
        while response.lower() == "n":
            self.new_task_menu(TODAY_STR)
        while response == "":
            self.timer_loop(timer)


def virgin_boostrap():
    global initial_boot_flag
    initial_boot_flag = True
    logger.info("First boot")
    logger.info(f"No data file found, creating at {DATA_FILE_LOCATION}")
    run_subprocess(HOME_DIR_CREATE)
    run_subprocess(DATA_FILE_CREATE)
    run_subprocess(TASK_FILE_CREATE)


def config_allocation(args, config, durations: Durations):
    config_file = ConfigFile()
    config_file.set_config(config, durations)
    logger.info(config.theme_color)
    if args.theme:
        config.theme_color = args.theme
    logger.info(config.theme_color)
    display.theme = display.color_map(config.theme_color)
    logger.info(display.theme)
    if args.work_time:
        durations.work.mins = args.work_time
        durations.work.secs = mins_to_secs(args.work_time)
    if args.rest_time:
        durations.rest.mins = args.rest_time
        durations.rest.secs = mins_to_secs(args.rest_time)


def run(args, config):
    global is_speed_mode, timer_mode

    durations_data = Durations()

    config_allocation(args, config, durations_data)

    if check_for_file(DATA_FILE_LOCATION) is False:
        virgin_boostrap()
    clear_screen()
    data_file.data_load_on_boot(durations_data)

    logger.info("Durations model set")
    logger.info(f"Work time: {durations_data.work.mins}")
    logger.info(f"Work time: {durations_data.work.secs}")
    logger.info(f"Work time: {durations_data.rest.mins}")
    logger.info(f"Work time: {durations_data.rest.secs}")

    if args.speed_mode is True:
        is_speed_mode = True
    logger.info(f"Speed mode set to {is_speed_mode}")

    if args.audio_file:
        sound = Sound(audio_path=args.audio_file)
        logger.debug(f"Custom audio file path specified {sound}")
    else:
        sound = Sound()
        logger.debug("Using default audio")
    sound.initialise()
    logger.debug("Sound initialised")
    sylo = Sylo(
        durations=durations_data,
        sound_obj=sound,
        mode=timer_mode,
        speed_mode=is_speed_mode,
    )
    sylo.input_loop()



import argparse


def get_arguments():
    parser = argparse.ArgumentParser(
        description="Sort Your Life Out! Python Pomodoro Timer"
    )
    parser.add_argument("-w", "--work_time", help="Set the work time length", type=int)
    parser.add_argument("-r", "--rest_time", help="Set the rest time length", type=int)
    parser.add_argument(
        "-a",
        "--audio_file",
        help="""Set the full path to an audio \
        file you want to be played on completion of a timer""",
        type=str,
    )
    parser.add_argument(
        "-l",
        "--log",
        help="Set the log level",
        type=str,
        choices=[
            "DEBUG",
            "INFO",
        ],
    )
    parser.add_argument("-s", "--speed_mode", dest="speed_mode", action="store_true")
    parser.add_argument(
        "-t",
        "--theme",
        help="Set the color scheme",
        type=str,
        choices=[
            "blue",
            "red",
            "green",
            "lblue",
            "lred",
            "lgreen",
            "magenta",
            "yellow",
            "white",
            "black",
        ],
    )
    parser.set_defaults(speed_mode=False)
    return parser.parse_args()



import logging
from sylo.args import get_arguments
from sylo.sylo import run
from sylo.sylo_logging import logging_config
from sylo.models import Config

logger = logging.getLogger(__name__)

config = Config()


def main():
    args = get_arguments()
    logging_config(args.log)
    run(args, config)
    logger.info("Started from __main__")


if __name__ == "__main__":

    main()



import sys
import logging
from sylo.definitions import ROOT_DIR

logger = logging.getLogger(__name__)


LOG_FILE_NAME = f"{ROOT_DIR}/../sylo.log"
LOG_FILE_FORMAT = "%(asctime)s %(module)s %(levelname)s: %(message)s"
DEFAULT_LOG_LEVEL = logging.WARN


def logging_config(log_arg: str = None):
    if log_arg:
        log_level = log_arg
        handler = logging.StreamHandler(sys.stdout)
        logging.basicConfig(level=log_level, filename=LOG_FILE_NAME, filemode="w")

    else:
        log_level = DEFAULT_LOG_LEVEL
        handler = None
        logging.basicConfig(
            level=log_level,
        )

    if handler:
        logging.getLogger().addHandler(handler)



import os
import datetime
from pathlib import Path


HOME_DIR = home = str(Path.home())
ROOT_DIR = os.path.dirname(os.path.abspath(__file__))

TODAY = datetime.datetime.now().date()
YESTERDAY = (datetime.datetime.now() - datetime.timedelta(days=1)).date()

TODAY_STR = TODAY.strftime("%Y-%m-%d")
YESTERDAY_STR = YESTERDAY.strftime("%Y-%m-%d")

ONE_WEEK_AGO = (datetime.datetime.now() - datetime.timedelta(days=7)).date()
ONE_YEAR_AGO = (datetime.datetime.now() - datetime.timedelta(days=364)).date()

DATA_FILE_LOCATION = f"{HOME_DIR}/.sylo/sessions.dat"
TASK_FILE_LOCATION = f"{HOME_DIR}/.sylo/tasks.dat"
CONFIG_FILE_LOCATION = f"{HOME_DIR}/.sylo/sylo.cfg"

HOME_DIR_CREATE = f"mkdir {HOME_DIR}/.sylo"
DATA_FILE_CREATE = f'echo "{TODAY_STR},0,0" > {DATA_FILE_LOCATION}'
TASK_FILE_CREATE = f"touch {TASK_FILE_LOCATION}"

WELCOME_CHOICES = ("", "h", "q", "s", "i", "t", "n", "y", "a")

TIMER_DEFAULTS = {
    "work": {
        "mins": 25,
        "secs": 1500,
    },
    "rest": {
        "mins": 5,
        "secs": 300,
    },
}

COUNTDOWN_INCREMENTS = 1

METRICS_HEAT = 'termgraph --calendar --start-dt "%s" %s '

METRICS_BAR = 'termgraph %s --color {red,green} --start-dt "%s"'



import os
import logging
import subprocess
import sys

logger = logging.getLogger(__name__)


def get_input(text):
    return input(text)


def run_subprocess(script: str):
    if "rm " in script:
        logger.warning("UNSAFE SCRIPT")
        sys.exit()
    try:
        subprocess.run(script, shell=True, check=True, timeout=30)
    except subprocess.CalledProcessError as e:
        print(e.output)
        logger.warning(e.output)


def clear_screen():
    os.system("cls" if os.name == "nt" else "clear")


def mins_to_secs(minutes: int):
    return minutes * 60


def flip_mode(mode: str):
    if mode == "work":
        logger.debug(f"flip_mode took {mode} returned rest")
        return "rest"
    else:
        logger.debug(f"flip_mode took {mode} returned work")
        return "work"


def check_for_file(path):
    is_file = os.path.isfile(path)
    logger.info(f"check_for_file {path} {is_file}")
    return is_file



import logging
from dataclasses import dataclass
import pandas as pd
from sylo.definitions import (
    TIMER_DEFAULTS,
    DATA_FILE_LOCATION,
    TODAY_STR,
    TASK_FILE_LOCATION,
    CONFIG_FILE_LOCATION,
)
from sylo.utils import check_for_file, mins_to_secs
import tomli
import csv

logger = logging.getLogger(__name__)


@dataclass
class Rest:
    mins: int = TIMER_DEFAULTS["rest"]["mins"]
    secs: int = TIMER_DEFAULTS["rest"]["secs"]
    bar_color: str = "green"


@dataclass
class Work:
    mins: int = TIMER_DEFAULTS["work"]["mins"]
    secs: int = TIMER_DEFAULTS["work"]["secs"]
    bar_color: str = "red"


@dataclass
class Durations:
    work = Work
    rest = Rest
    total_work_mins: float = 0
    total_rest_mins: float = 0

    def __post_init__(self):
        self.work = Work()
        self.rest = Rest()


@dataclass
class Config:
    theme_color = "blue"
    work_time = 25
    rest_time = 5


class ConfigFile:
    def __init__(self):
        self.config_file = CONFIG_FILE_LOCATION
        self.is_config = check_for_file(self.config_file)

    def load_config(self):
        with open(self.config_file, encoding="utf-8") as f:
            return tomli.load(f)

    def set_config(self, config: Config, durations_model: Durations):
        if self.is_config is True:
            config_file = self.load_config()
            logger.info(config_file)
            try:
                general = config_file["general"]
                logger.info(f"CONFIG: general from file: {general}")
                config.theme_color = general["theme"]
            except KeyError as k:
                logger.info(k)
                pass

            try:
                durations = config_file["durations"]
                logger.info(f"CONFIG: durations from file: {durations}")
                durations_model.work.mins = durations["work"]
                durations_model.work.secs = mins_to_secs(durations["work"])
                durations_model.rest.mins = durations["work"]
                durations_model.rest.secs = mins_to_secs(durations["rest"])

            except KeyError as k:
                logger.info(k)
                pass


class DataPersistence:
    def __init__(self, data_file: str = DATA_FILE_LOCATION):
        self.data_file = data_file
        self.file_cols = ["date", "time_worked", "time_rested"]

    def data_refresh(self, durations: Durations):
        new_df = self.fetch_and_update_values(durations, TODAY_STR)
        new_df.to_csv(DATA_FILE_LOCATION, sep=",", mode="w", header=False)

    def data_load_on_boot(self, durations: Durations):
        df = self._read_file()
        try:
            filtered_dates = df.loc[TODAY_STR]
            logger.info("data_load_on_boot Found data for today")
            work_time_today = float(filtered_dates["time_worked"])
            rest_time_today = float(filtered_dates["time_rested"])
            logger.info(f"Work time loaded on boot: {work_time_today}")
            logger.info(f"Rest time loaded on boot: {rest_time_today}")
            durations.total_work_mins = work_time_today
            durations.total_rest_mins = rest_time_today
        except Exception:
            logger.info("data_load_on_boot NO DATA for today")
            filtered_dates = None
        if filtered_dates is None:
            durations.total_rest_mins = 0
            durations.total_work_mins = 0

    def _read_file(self):
        df = pd.read_csv(self.data_file, header=None, sep=",")
        df.columns = self.file_cols
        df.set_index("date", inplace=True, drop=True)
        return df

    def fetch_and_update_values(self, durations: Durations, date: str = TODAY_STR):
        df = self._read_file()
        new_df = pd.DataFrame(
            {
                "date": [date],
                "time_worked": [durations.total_work_mins],
                "time_rested": [durations.total_rest_mins],
            },
            columns=self.file_cols,
            dtype="float",
        )
        new_df.set_index("date", inplace=True, drop=True)
        out = new_df.combine_first(df)
        return out


def get_tasks(date: str = None):
    try:
        with open(TASK_FILE_LOCATION, "r") as read_obj:
            csv_reader = csv.reader(read_obj)
            logger.info(f"Returning tasks for {date}")
            if date:
                return [row for row in csv_reader if row[0] == date]
            else:
                return [row for row in csv_reader]
    except Exception as e:
        print(e)
